#include "mcutil/collect/gen_hash_map.h"
#include <assert.h>
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include "mcutil/mcutil.h"
#include "mcutil/hash/md5.h"

#define CREATE_HM_NODES_IF_NULL(hm) { if (!(hm)->nodes) (hm)->nodes = MCU_ALLOC(sizeof(mcu_gen_hm_node_t)); }

MCU_API mcu_gen_hm_t mcu_hm_new() {
    return (mcu_gen_hm_t){
        .nodes = MCU_ALLOC(sizeof(mcu_gen_hm_node_t)),
    };
}

MCU_API void _mcu_hm_insert(mcu_gen_hm_t* hm, const void* key, const size_t key_len, const void* value) {
    assert(hm && "HashMap ptr is null");
    mcu_md5_context ctx = {0};
    mcu_md5_init(&ctx);
    mcu_md5_digest(&ctx, (void*)key, key_len);
    uint8_t key_res[16];
    mcu_md5_output(&ctx, key_res);
    
    CREATE_HM_NODES_IF_NULL(hm);

    mcu_gen_hm_node_t* node = hm->nodes;
    while (true) {
        if (!node->next) {
            node->next = MCU_ALLOC(sizeof(mcu_gen_hm_node_t));
            node->next->next = NULL;
            memcpy(node->next->key, key_res, 16);
            node->next->value = (void*)value;
            return;
        }
        node = node->next;
    }
}

MCU_API void* _mcu_hm_get(mcu_gen_hm_t* hm, const void* key, const size_t key_len) {
    assert(hm && "HashMap ptr is null");
    mcu_md5_context ctx = {0};
    mcu_md5_init(&ctx);
    mcu_md5_digest(&ctx, (void*)key, key_len);
    uint8_t key_res[16];
    mcu_md5_output(&ctx, key_res);
    
    CREATE_HM_NODES_IF_NULL(hm);
    mcu_gen_hm_node_t* node = hm->nodes;
    while (true) {
        if (memcmp(node->key, key_res, 16) == 0) {
            return node->value;
        } else {
            if (!node->next) return NULL;
            node = node->next;
        }
    }
}

MCU_API void* _mcu_hm_remove(mcu_gen_hm_t* hm, const void* key, const size_t key_len) {
    assert(hm && "HashMap ptr is null");
    mcu_md5_context ctx = {0};
    mcu_md5_init(&ctx);
    mcu_md5_digest(&ctx, (void*)key, key_len);
    uint8_t key_res[16];
    mcu_md5_output(&ctx, key_res);
    
    CREATE_HM_NODES_IF_NULL(hm);

    mcu_gen_hm_node_t* node = hm->nodes;
    mcu_gen_hm_node_t* prev = hm->nodes;
    while (true) {
        if (memcmp(node->key, key_res, 16) == 0) {
            void* val = node->value;
            prev->next = node->next;
            return val;
        } else {
            if (!node->next) return NULL;
            prev = node;
            node = node->next;
        }
    }
}

MCU_API void mcu_free(mcu_gen_hm_t* hm) {
    assert(hm && "HashMap ptr is null");
    mcu_gen_hm_node_t* node = hm->nodes;
    if (!node) return;

    while (true) {
        if (node->next) return;
        MCU_FREE(node);
        node = node->next;
    }
}

MCU_API void mcu_free_all(mcu_gen_hm_t* hm) {
    assert(hm && "HashMap ptr is null");
    mcu_gen_hm_node_t* node = hm->nodes;
    if (!node) return;
    while (true) {
        if (node->next) return;
        MCU_FREE(node);
        MCU_FREE(node->value);
        node = node->next;
    }
}
